{            IMPLEMENTATION section include file for Comm_TP5.PAS.            } {-This is the comment column }
{                                                                             } { which explains the program }
{         Comm_IO5.PAS Ver. 2.50 - RS-232 Support for IBM Compatibles         } { operation where necessary. }
{                       (c) Copyright, 1989, 1990, 1991                       } {                            }
{                              Kevin R. Bulgrien                              } { It is likely to cause your }
{                               January 27, 1999                              } { hard copy to exceed a line }
{                                                                             } { width of 80 characters, so }
{ See the accompanying file Comm_TP5.DOC of specific information regarding    } { your printer will need to  }
{ distribution policies & supplementary documentation for the source code.    } { be set for smaller print.  }
{                                                                             } {                            }
{ Written by:  Kevin R. Bulgrien           Version 2.50 completed 08/06/91    } { I hope this is acceptably  }
{                                                                             } { inconvenient as I believe  }
{ Send all correspondance to:              Kevin R. Bulgrien                  } { that documentation is far  }
{                                          P. O. Box 563                      } { easier to read when it is  }
{ CompuServe: 102174, 1561                 Longview, TX 75606                 } { not mingled with the code. }
{ Internet:   kbulgrien@worldnet.att.net   (903) 757-5163                     } {                            }
{                                                                             } { No code should be placed   }
{ Also contact via: c/o K. A. Bulgrien, 1700 Sheldon Road, Sandusky, MI 48471 } { in this column so that it  }
{                                                                             } { will be easier to remove.  }
{ This code package must always include the following files:  Comm_TP5.PAS,   } {                            }
{ Comm_TP5.H, Comm_TP5.ASM, Comm_TPA.OBJ, Comm_TPB.OBJ, Comm_TP5.DOC,         } {-When Comm_TPx.OBJ is here, }
{ Comm_IO5.PAS, Comm_IO5.H, and Comm_TTY.PAS.                                 } { no need for an assembler.  }
                                                                                {                            }
FUNCTION WriteCOM (ID : SHORTINT; Data : STRING) : BOOLEAN;                     {-Send data to a port while  }
{$IFDEF ErrorChecking}                                                          { waiting for confirmation   }
  CONST ProcName = 'WriteCOM';                                                  { that the data could be     }
{$ENDIF}                                                                        { sent.                      }
VAR                                                                             {                            }
  Loop : WORD;                                                                  {-Pointer into output string }
  TimeLeft : BOOLEAN;                                                           {-Transmission status        }
BEGIN                                                                           {                            }
  TimeLeft := TRUE;                                                             {-Default of no errors       }
  {$IFDEF ErrorChecking}                                                        {                            }
    IF NOT ValidPort (ID, ProcName)                                             {-Optional error trap checks }
      THEN TimeLeft := FALSE                                                    { to see if the serial port  }
      ELSE                                                                      { specified was installed.   }
  {$ENDIF}                                                                      {                            }
        WITH PortInfo [ID]^ DO                                                  {                            }
          BEGIN                                                                 {                            }
            Loop := 1;                                                          {-Point to first character   }
            DisableInts;                                                        {                            }
            O.Time := 0;                                                        {-Clear the port timer       }
            EnableInts;                                                         {                            }
            WHILE (Loop <= LENGTH (Data)) AND TimeLeft DO                       {-Loop until all data has    }
              BEGIN                                                             { been sent, or until too    }
                DisableInts;                                                    { much time has passed.      }
                TimeLeft := (O.Time <= O.MaxTime);                              { Note use of temporaries!   }
                EnableInts;                                                     {                            }
                IF TimeLeft                                                     {                            }
                  THEN                                                          {                            }
                    BEGIN                                                       {                            }
                      DisableInts;                                              {                            }
                      IF ((PORT [COMAddr + LSR] AND $20) <> 0)                  {-Check the THR to see if it }
                        THEN                                                    { is ready for one character }
                          BEGIN                                                 { to send.  If so, put the   }
                            PORT [COMAddr + THR] := ORD (Data [Loop]);          { data in the THR. If not,   }
                            INC (Loop);                                         { wait a bit and try again.  }
                          END;                                                  {                            }
                      EnableInts;                                               {                            }
                    END;                                                        {                            }
              END;                                                              {                            }
            {$IFDEF ErrorChecking}                                              {-Optionally generate an     }
              IF NOT TimeLeft                                                   { error condition if the     }
                THEN MakeError (3, ID, ProcName);                               { port times out.            }
            {$ENDIF}                                                            {                            }
          END;                                                                  {                            }
  WriteCOM := TimeLeft;                                                         {-Return TRUE if all data    }
END;                                                                            { was sent in time.          }
                                                                                {                            }
FUNCTION IWriteCOM (ID : SHORTINT; Data : STRING) : BOOLEAN;                    {-Send data to a port and do }
{$IFDEF ErrorChecking}                                                          { not wait for confirmation  }
  CONST ProcName = 'IWriteCOM';                                                 { that the data was sent.    }
{$ENDIF}                                                                        {                            }
VAR                                                                             {                            }
  HowMany, Loop : WORD;                                                         {-Temporary variables        }
  TimeLeft : BOOLEAN;                                                           {-Transmission status        }
BEGIN                                                                           {                            }
  TimeLeft := TRUE;                                                             {-Default of no errors       }
  {$IFDEF ErrorChecking}                                                        {                            }
    IF NOT ValidPort (ID, ProcName)                                             {-Optional error trap checks }
      THEN TimeLeft := FALSE                                                    { to see if the serial port  }
      ELSE                                                                      { specified was installed.   }
  {$ENDIF}                                                                      {                            }
        WITH PortInfo [ID]^ DO                                                  {                            }
          BEGIN                                                                 {                            }
            Loop := 1;                                                          {-Point to first character   }
            DisableInts;                                                        {                            }
            O.Time := 0;                                                        {-Clear the port timer       }
            EnableInts;                                                         {                            }
            WHILE (Loop <= LENGTH (Data)) AND TimeLeft DO                       {-Loop until all data has    }
              BEGIN                                                             { been sent, or until too    }
                DisableInts;                                                    { much time has passed.      }
                TimeLeft := (O.Time <= O.MaxTime);                              { Note use of temporaries!   }
                HowMany := O.Size;                                              { See DOC file for details.  }
                EnableInts;                                                     {                            }
                IF TimeLeft AND (HowMany < O.MaxSize)                           {-Only try to send data if   }
                  THEN                                                          { the buffer has room for    }
                    BEGIN                                                       { the characters.            }
                      DisableInts;                                              {                                 }
                      O.Buffer^ [O.Tail] := ORD (Data [Loop]);                  {-Put the character in the   }
                      O.Tail := (O.Tail + 1) MOD O.MaxSize;                     { buffer & update pointers.  }
                      INC (O.Size);                                             {                            }
                      EnableInts;                                               {                            }
                      INC (Loop);                                               {                            }
                    END;                                                        {                            }
              END;                                                              {                            }
            {$IFDEF ErrorChecking}                                              {-Optionally generate an     }
              IF NOT TimeLeft                                                   { error condition if the     }
                THEN MakeError (3, ID, ProcName);                               { port times out.            }
            {$ENDIF}                                                            {                            }
          END;                                                                  {                            }
  IWriteCOM := TimeLeft;                                                        {-Return TRUE if all data    }
END;                                                                            { was sent in time.          }
                                                                                {                            }
FUNCTION ReadCOM (ID : SHORTINT; VAR Data : CHAR) : BOOLEAN;                    {-Read data directly from a  }
{$IFDEF ErrorChecking}                                                          { serial port.               }
  CONST ProcName = 'ReadCOM';                                                   {                            }
{$ENDIF}                                                                        {                            }
VAR                                                                             { Temporary & Transmission   }
  NoData, TimeLeft : BOOLEAN;                                                   {-status variables.          }
BEGIN                                                                           {                            }
  TimeLeft := TRUE;                                                             {-Default of no errors       }
  {$IFDEF ErrorChecking}                                                        {-Optional error trap checks }
    IF NOT ValidPort (ID, ProcName)                                             { to see if the serial port  }
      THEN TimeLeft := FALSE                                                    { specified was installed.   }
      ELSE                                                                      {                            }
  {$ENDIF}                                                                      {                            }
        WITH PortInfo [ID]^ DO                                                  {                            }
          BEGIN                                                                 {                            }
            NoData := TRUE;                                                     {-Default: No data received  }
            DisableInts;                                                        {                            }
            I.Time := 0;                                                        {                            }
            EnableInts;                                                         {                            }
            WHILE NoData AND TimeLeft DO                                        {-Loop until data has been   }
              BEGIN                                                             { received, or until too     }
                DisableInts;                                                    {                            }
                TimeLeft := (I.Time <= I.MaxTime);                              { Note use of temporaries!   }
                EnableInts;                                                     {                            }
                IF TimeLeft                                                     {                            }
                  THEN                                                          { If time has not run out    }
                    BEGIN                                                       { yet, check the LSR to see  }
                      DisableInts;                                              { if a complete character    }
                      IF ((PORT [COMAddr + LSR] AND $01) <> 0)                  {-is waiting in the RHR.     }
                        THEN                                                    {                            }
                          BEGIN                                                 {                            }
                            Data := CHR (PORT [COMAddr + RHR]);                 {-If data was ready, get it  }
                            NoData := FALSE;                                    { and signal that we have    }
                          END;                                                  { read the character.        }
                      EnableInts;                                               {                            }
                    END;                                                        {                            }
              END;                                                              {                            }
            {$IFDEF ErrorChecking}                                              {-Optionally generate an     }
              IF NOT TimeLeft                                                   { error condition if the     }
                THEN MakeError (3, ID, ProcName);                               { port times out.            }
            {$ENDIF}                                                            {                            }
          END;                                                                  {                            }
  ReadCOM := TimeLeft;                                                          {-Return TRUE if data was    }
END;                                                                            { was received in time.      }
                                                                                {                            }
FUNCTION IReadCOM (ID : SHORTINT; VAR Data : CHAR) : BOOLEAN;                   {-Read data received in the  }
{$IFDEF ErrorChecking}                                                          { background that was put    }
  CONST ProcName = 'IReadCOM';                                                  { in a port buffer.          }
{$ENDIF}                                                                        {                            }
VAR                                                                             {                            }
  TimeOut : BOOLEAN;                                                            {-Transmission Status        }
  HowMany : WORD;                                                               {-Temporary buffer status    }
BEGIN                                                                           {                            }
  TimeOut := TRUE;                                                              {-Default of no errors       }
  {$IFDEF ErrorChecking}                                                        {-Optional error trap checks }
    IF NOT ValidPort (ID, ProcName)                                             { to see if the serial port  }
      THEN TimeOut := TRUE                                                      { specified was installed.   }
      ELSE                                                                      {                            }
  {$ENDIF}                                                                      {                            }
        WITH PortInfo [ID]^ . I DO                                              {                            }
          BEGIN                                                                 {                            }
            DisableInts;                                                        {                            }
            Time := 0;                                                          {                            }
            EnableInts;                                                         {                            }
            REPEAT                                                              {-Loop until a character is  }
              DisableInts;                                                      {                            }
              TimeOut := (Time > MaxTime);                                      { Note use of temporaries!   }
              HowMany := Size;                                                  { use of temporaries here!   }
              EnableInts;                                                       { See DOC file for more on   }
            UNTIL (HowMany <> 0) OR TimeOut;                                    { why this method is used.   }
            IF (HowMany <> 0) AND NOT TimeOut                                   {                            }
              THEN                                                              { If there is data in the    }
                BEGIN                                                           { buffer, take one character }
                  DisableInts;                                                  { out and adjust the buffer  }
                  Data := CHR (Buffer^ [Head]);                                 {-pointers and status vars.  }
                  Head := (Head + 1) MOD MaxSize;                               {                            }
                  DEC (Size);                                                   {                            }
                  EnableInts;                                                   {                            }
                END;                                                            {                            }
            {$IFDEF ErrorChecking}                                              {-Optionally generate an     }
              IF TimeOut                                                        { error condition if the     }
                THEN MakeError (4, ID, ProcName);                               { port times out.            }
            {$ENDIF}                                                            {                            }
          END;                                                                  {                            }
  IReadCOM := NOT TimeOut;                                                      {-Return TRUE if data was    }
END;                                                                            { was received in time.      }
                                                                                {                            }
FUNCTION IReadCOMStr (ID:SHORTINT; Chars:BYTE; VAR Data:STRING) : BOOLEAN;      {-Read a string received in  }
{$IFDEF ErrorChecking}                                                          { the background that was    }
  CONST ProcName = 'IReadCOMStr';                                               { put in a port buffer.      }
{$ENDIF}                                                                        {                            }
VAR                                                                             {                            }
  TimeOut : BOOLEAN;                                                            {-Transmission Status        }
  HowMany : WORD;                                                               {-Number of chars in buffer  }
  Last : CHAR;                                                                  {-Last processed character   }
BEGIN                                                                           {                            }
  TimeOut := FALSE;                                                             {-Default of no errors       }
  {$IFDEF ErrorChecking}                                                        {-Optional error trap checks }
    IF NOT ValidPort (ID, ProcName)                                             { to see if the serial port  }
      THEN TimeOut := TRUE                                                      { specified was installed.   }
      ELSE                                                                      {                            }
  {$ENDIF}                                                                      {                            }
        WITH PortInfo [ID]^ . I DO                                              {                            }
          BEGIN                                                                 {                            }
            Data := '';                                                         {-Clear the receive string   }
            DisableInts;                                                        {                            }
            Time := 0;                                                          {                            }
            EnableInts;                                                         {                            }
            REPEAT                                                              {                            }
              REPEAT                                                            {-Loop until a character is  }
                DisableInts;                                                    {                            }
                TimeOut := (Time > MaxTime);                                    { Note use of temporaries!   }
                HowMany := Size;                                                { use of temporaries here!   }
                EnableInts;                                                     { See DOC file for details   }
              UNTIL (HowMany <> 0) OR TimeOut;                                  { why this method is used.   }
              IF (HowMany <> 0) AND NOT TimeOut                                 {                            }
                THEN                                                            { If there is data in the    }
                  BEGIN                                                         { buffer, take one character }
                    DisableInts;                                                { out and adjust the buffer  }
                    Last := CHR (Buffer^ [Head]);                               {-pointers and status vars.  }
                    Head := (Head + 1) MOD MaxSize;                             {                            }
                    DEC (Size);                                                 { Wait for the right number  }
                    EnableInts;                                                 { of characters or for time  }
                    Data := Data + Last;                                        { to run out.  If no number  }
                  END;                                                          { of characters is chosen,   }
            UNTIL (LENGTH (Data) = Chars)                                       {-wait for CHR (12).         }
            OR ((Chars = 0) AND (Last = #12)) OR TimeOut;                       {                            }
            {$IFDEF ErrorChecking}                                              {-Optionally generate an     }
              IF TimeOut                                                        { error condition if the     }
                THEN MakeError (4, ID, ProcName);                               { port times out.            }
            {$ENDIF}                                                            {                            }
          END;                                                                  {                            }
  IReadCOMStr := NOT TimeOut;                                                   {-Return TRUE if data was    }
END;                                                                            { was received in time.      }
