UNIT Comm_TP5;                                                                  {                            }
                                                                                {                            }
{                     Turbo Pascal (Version 5.0 and higher)                   } {-This is the comment column }
{                                                                             } { which explains the program }
{         Comm_TP5.PAS Ver. 2.50 - RS-232 Support for IBM Compatibles         } { operation where necessary. }
{                       (c) Copyright, 1989, 1990, 1991                       } {                            }
{                              Kevin R. Bulgrien                              } { It is likely to cause your }
{                               January 27, 1999                              } { hard copy to exceed a line }
{                                                                             } { width of 80 characters, so }
{ See the accompanying file Comm_TP5.DOC of specific information regarding    } { your printer will need to  }
{ distribution policies & supplementary documentation for the source code.    } { be set for smaller print.  }
{                                                                             } {                            }
{ Written by:  Kevin R. Bulgrien           Version 2.50 completed 08/06/91    } { I hope this is acceptably  }
{                                                                             } { inconvenient as I believe  }
{ Send all correspondance to:              Kevin R. Bulgrien                  } { that documentation is far  }
{                                          P. O. Box 563                      } { easier to read when it is  }
{ CompuServe: 102174,1561                  Longview, TX 75606                 } { not mingled with the code. }
{ Internet:   kbulgrien@worldnet.att.net   (903) 757-5163                     } {                            }
{                                                                             } { No code should be placed   }
{ Also contact via: c/o K. A. Bulgrien, 1700 Sheldon Road, Sandusky, MI 48471 } { in this column so that it  }
{                                                                             } { will be easier to remove.  }
{ This code package must always include the following files:  Comm_TP5.PAS,   } {                            }
{ Comm_TP5.H, Comm_TP5.ASM, Comm_TPA.OBJ, Comm_TPB.OBJ, Comm_TP5.DOC,         } {-When Comm_TPx.OBJ is here, }
{ Comm_IO5.PAS, Comm_IO5.H, and Comm_TTY.PAS.                                 } { no need for an assembler.  }
                                                                                {                            }
{$B-} { I almost always plan on using short-circuit expression when possible  } {-Will not work otherwise!   }
{$I-} { I/O checking is unnecessary since all we do is write to the screen    } {-You may turn Debug Info &  }
{$D-} { Debugging information is unnecessary, and only bloats the .TPU file   } { IO or Range checks on, but }
{$R-} { Range checking is unnecessary and makes the interrupt handler slower  } { watch the .TPU get HUGE!   }
{$S-} { Interrupt handlers should not be compiled with stack checking enabled } {-Else the system may crash! }
                                                                                {                            }
{$DEFINE ErrorChecking}                                                         {-Enable/Disable Error Check }
{$UNDEF  NoErrorMsgs}                                                           {-Enable/Disable Error Msgs  }
{$DEFINE ASMInt}                                                                {-Enable/Disable assembly    }
                                                                                { language version of        }
INTERFACE                                                                       { SerialInt.                 }
                                                                                {                            }
USES DOS, CRT;                                                                  { These C style header files }
                                                                                { make it easier for me to   }
{$I Comm_TP5.H}  { Contains global declarations accessible to other programs. } {-more easily translate the  }
{$I Comm_IO5.H}  { Global I/O routine declarations. }                           { routines back and forth    }
                                                                                { from Comm_TC source code.  }
IMPLEMENTATION                                                                  {                            }
                                                                                { Comm_TP5 will crash if the }
{$F+}                                                                           {-they are not declared FAR. }
{$IFDEF ASMInt}                                                                 {                            }
  {$L Comm_TPA.OBJ}                                                             {-Load OBJ file with serial  }
  PROCEDURE SerialInt; INTERRUPT; EXTERNAL;                                     { & timer interrupt support. }
{$ELSE}                                                                         {                            }
  {$L Comm_TPB.OBJ}                                                             {-Load only timer interrupt. }
{$ENDIF}                                                                        {                            }
PROCEDURE InstallTimer; EXTERNAL;                                               {-Activates Comm_TP5 timer   }
PROCEDURE RemoveTimer; EXTERNAL;                                                {-Disables Comm_TP5 timer    }
{$F-}                                                                           {                            }
                                                                                {                            }
VAR                                                                             {                            }
  OldIntVector : ARRAY [0..MaxPorts - 1] OF POINTER;                            {-Original DOS int. vectors  }
  ExitSave : POINTER;                                                           {-Saves EXITPROC pointer     }
  Loop : SHORTINT;                                                              {-General purpose loop       }
  IRQMask : BYTE;                                                               {-IRQMask tracks IRQs in use }
                                                                                {                            }
{$IFDEF ErrorChecking}                                                          {                            }
PROCEDURE MakeError (Code : BYTE; Port : SHORTINT; ProcName : ProcNameType);    {-Generate a Comm_TP5 error  }
BEGIN                                                                           {                            }
  ErrorCode := Code;                                                            {-Set the global error type  }
  ErrorPort := Port;                                                            { and port variables.        }
  {$IFNDEF NoErrorMsgs}                                                         {                            }
    IF ShowMessages                                                             {-Should messages be shown?  }
      THEN                                                                      { Only if ShowMessages TRUE. }
        BEGIN                                                                   {                            }
          IF (ErrMsgX > 0) THEN GOTOXY (ErrMsgX, WHEREY);                       {-Print error messages. Zero }
          IF (ErrMsgY > 0) THEN GOTOXY (WHEREX, ErrMsgY);                       { coordinates cause messages }
          WRITE (ProcName, ' ERROR: ');                                         { to print at current cursor.}
          CASE Code OF                                                          {                            }
             1 : WRITE ('Invalid port # ', Port);                               { 0 <= Good Port < MaxPorts  }
             2 : WRITE ('Port # ', Port, ' not installed yet');                 {-Port must be installed!    }
             3 : WRITE ('Timeout writing port # ', Port);                       {-WriteCOM error             }
             4 : WRITE ('Timeout reading port # ', Port);                       {-TimedReadCOM error         }
             5 : WRITE ('Invalid interrupt mode requested');                    {-InstallInt error           }
             6 : WRITE ('Invalid IRQ number requested');                        {-InstallInt error           }
             7 : WRITE ('COM', Port, ' already installed');                     {-Use InstallInt once/port   }
             8 : WRITE ('Too many port handlers requested');                    {-MaxPorts is the limit      }
             9 : WRITE ('Invalid buffer size for port handler # ', Port);       {-InstallInt error           }
            10 : WRITE ('Not enough memory for port handler # ', Port);         {-InstallInt can't get memory}
            11 : WRITE ('Invalid port parameters (Port #', Port, ')');          {-Bad SetCOMPort parameter   }
            12 : WRITE ('Invalid RTS/DTR Setting (Port #', Port, ')');          {-Invalid SetRtsDtr params   }
          END;                                                                  {-These error messages are   }
        END;                                                                    { displayed only when they   }
  {$ENDIF}                                                                      { are compiled with $UNDEF   }
END;                                                                            { NoErrorMsgs.               }
                                                                                {                            }
FUNCTION ValidPort (Port : SHORTINT; ProcName : ProcNameType) : BOOLEAN;        {-Checks if port # is valid  }
BEGIN                                                                           {                            }
  ErrorCode := 0;                                                               {-Default of no errors       }
  IF (Port < 0) OR (Port >= MaxPorts)                                           {-Check if port # is valid   }
    THEN MakeError (1, Port, ProcName)                                          {-If not, create an error    }
    ELSE                                                                        {                            }
      IF (PortInfo [Port] = NIL)                                                {-Is port installed yet?     }
        THEN MakeError (2, Port, ProcName);                                     {-If not, create an error    }
  ValidPort := (ErrorCode = 0);                                                 {-Return TRUE if no errors   }
END;                                                                            {                            }
{$ENDIF}                                                                        {                            }
                                                                                {                            }
{$F+}                                                                           {-Interrupt handlers MUST be }
{$IFNDEF ASMInt}                                                                { FAR calls.  Compiles only  }
PROCEDURE SerialInt; INTERRUPT;                                                 { if ASM version unavailable.}
VAR                                                                             {                            }
  IRQ,                                                                          {-IRQ # of this interrupt    }
  IMR,                                                                          {-Interrupt Mask Register    }
  Temp : BYTE;                                                                  {-Temporary for misc. uses   }
  ID : SHORTINT;                                                                {-Port being serviced        }
BEGIN                                                                           {                            }
  IRQ := 0;                                                                     {                            }
  IMR := PORT [$21];                                                            {-Store IMR for later use    }
  PORT [$20] := $0B;                                                            {-Request 8259 an ISR read   }
  Temp := PORT [$20];                                                           {-Get the 8259 ISR value     }
  WHILE (Temp AND 1 = 0) DO                                                     {-Upon loop completion, IRQ  }
    BEGIN                                                                       { will contain the IRQ # of  }
      Temp := Temp SHR 1;                                                       { the interrupt in progress. }
      INC (IRQ);                                                                {                            }
    END;                                                                        {                            }
  PORT [$21] := IMR OR IRQMask;                                                 {-Disable Comm_TP5 interrupts}
  EnableInts;                                                                   { but others are okay.       }
  FOR ID := 0 TO MaxPorts - 1 DO                                                {-Process all ports on the   }
    IF (PortInfo [ID] <> NIL) AND (PortInfo [ID]^ . IRQNmbr = IRQ)              { IRQ level which we are     }
      THEN                                                                      { processing.                }
        WITH PortInfo [ID]^ DO                                                  {                            }
          BEGIN                                                                 {                            }
            PORT [COMAddr + LCR] := PORT [COMAddr + LCR] AND $7F;               {-Set THR, RHR & IER active  }
            CASE PORT [COMAddr + IIR] AND $07 OF                                {-Identify interrupt type    }
              6 : BEGIN                                                         {                            }
                    Temp := PORT [COMAddr + LSR] AND $1E;                       { LINE STATUS CHANGE & ERROR }
                    IF (Temp AND $02 <> 0) THEN INC (Overrun);                  {   Just count the errors    }
                    IF (Temp AND $04 <> 0) THEN INC (Parity);                   {       Overrun Error        }
                    IF (Temp AND $08 <> 0) THEN INC (Framing);                  {       Parity Error         }
                    IF (Temp AND $10 <> 0) THEN INC (Break);                    {       Framing Error        }
                  END;                                                          {       Break Interrupt      }
              4 : BEGIN                                                         {                            }
                    IF ((I.Tail + 1) MOD I.MaxSize <> I.Head)                   {   RECEIVE REGISTER FULL    }
                      THEN                                                      {                            }
                        BEGIN                                                   { If the buffer is not full, }
                          I.Buffer^ [I.Tail] := PORT [COMAddr + RHR];           { add the character and set  }
                          I.Tail := (I.Tail + 1) MOD I.MaxSize;                 { the queue buffer pointer,  }
                          INC (I.Size);                                         { and mark addition of data. }
                        END                                                     {                            }
                      ELSE                                                      {                            }
                        BEGIN                                                   { Otherwise, the character   }
                          IF (PORT [COMAddr + RHR] = $00) THEN {Nothing};       {-is read but not stored.    }
                          OverFlow := TRUE;                                     {-A flag is set so that this }
                        END;                                                    { condition can be detected. }
                  END;                                                          {                            }
              2 : BEGIN                                                         {                            }
                    IF (O.Head = O.Tail)                                        {  TRANSMIT REGISTER EMPTY   }
                      THEN PORT [COMAddr+IER] := PORT [COMAddr+IER] AND $FD     {  If no more data to send,  }
                      ELSE                                                      {  shut off the transmitter. }
                        BEGIN                                                   {  Otherwise, send the next  }
                          PORT [COMAddr + THR] := O.Buffer^ [O.Head];           {  byte, and remove it from  }
                          O.Head := (O.Head + 1) MOD O.MaxSize;                 {  the buffer.  Mark the     }
                          DEC (O.Size);                                         {  deletion of data.         }
                        END;                                                    {                            }
                  END;                                                          {                            }
              0 : BEGIN                                                         {                            }
                    Temp := PORT [COMAddr + MSR];                               {    MODEM STATUS CHANGES    }
                    CD  := $80 AND Temp <> 0;                                   {       Carrier Detect       }
                    CTS := $10 AND Temp <> 0;                                   {       Clear To Send        }
                    DSR := $20 AND Temp <> 0;                                   {       Data Set Ready       }
                    RI  := $40 AND Temp <> 0;                                   {       Ring Indicator       }
                  END;                                                          {                            }
            END;                                                                {                            }
          END;                                                                  {                            }
  DisableInts;                                                                  {-Accessing 8259 hardware    }
  PORT [$21] := IMR;                                                            {-Enable Comm_TP5 interrupts }
  PORT [$20] := $20;                                                            {-Notify 8259 of interrupt   }
END;                                                                            { completion (nonspecific)   }
{$ENDIF}                                                                        {                            }
{$F-}                                                                           {                            }
                                                                                {                            }
FUNCTION InstallInt (Mode:BYTE;Com,IRQ:BYTE;Addr,ISize,OSize:WORD) : SHORTINT;  {-Installs a port handler &  }
CONST                                                                           { defines which hardware     }
  ProcName = 'InstallInt';                                                      { port it is serving.        }
VAR                                                                             {                            }
  SameIRQ, Temporary : BYTE;                                                    {-Temporary storage          }
  ID : SHORTINT;                                                                {-Port handler # to return   }
LABEL                                                                           {                            }
  Abort;                                                                        {-Some error checking always }
BEGIN                                                                           { is in place.  InstallInt   }
  ID := -1;                                                                     { is the most critical of    }
  IF (Mode > 15)                                                                { the Comm_TP5 routines.     }
    THEN                                                                        { Most of the parameters     }
      BEGIN                                                                     { are checked at least a     }
        {$IFDEF ErrorChecking}                                                  { little bit.  If obvious    }
          MakeError (5, ID, ProcName);                                          { errors occur, InstallInt   }
        {$ENDIF}                                                                { aborts with a -1 port ID.  }
        GOTO Abort;                                                             {                            }
      END;                                                                      {-Check Mode & IRQ parameter }
  IF (IRQ < 1) OR (IRQ > 7)                                                     { values.  Errors can still  }
    THEN                                                                        { slip by, but most obvious  }
      BEGIN                                                                     { ones will be caught.  IRQ  }
        {$IFDEF ErrorChecking}                                                  { is especially difficult to }
          MakeError (6, ID, ProcName);                                          { check, so be very careful  }
        {$ENDIF}                                                                { when specifying it. It is  }
        GOTO Abort;                                                             { usually only going to be   }
      END;                                                                      { 3 or 4, and sometimes 5.   }
  SameIRQ := 0;                                                                 {                            }
  FOR Temporary := MaxPorts - 1 DOWNTO 0 DO                                     {-Find an unused port number }
    BEGIN                                                                       { from the pool of available }
      IF (PortInfo [Temporary] = NIL)                                           { port handler pointers.     }
        THEN ID := Temporary                                                    {                            }
        ELSE                                                                    { If the same IRQ is used on }
          BEGIN                                                                 { another port, save port #  }
            IF (PortInfo [Temporary]^.IRQNmbr = IRQ)                            { so the int. vector is not  }
              THEN SameIRQ := IRQ;                                              {-set again.                 }
            IF (PortInfo [Temporary]^.COMAddr = Addr)                           {                            }
              THEN BEGIN                                                        {-The port base address is   }
                     {$IFDEF ErrorChecking}                                     { checked to be sure it is   }
                       MakeError (7, Com, ProcName);                            { not already installed.     }
                     {$ENDIF}                                                   { This is a limited check,   }
                     GOTO Abort;                                                { but again detects simple   }
                   END;                                                         { errors.                    }
          END;                                                                  {                            }
    END;                                                                        { ID is port handler to use, }
  IF (ID = -1)                                                                  { or -1 if none were unused, }
    THEN                                                                        {-in which case, InstallInt  }
      BEGIN                                                                     { is aborted.                }
        {$IFDEF ErrorChecking}                                                  {                            }
          MakeError (8, 0, ProcName);                                           {-The $DEFINE ErrorChecking  }
        {$ENDIF}                                                                { directive only disables    }
        GOTO Abort;                                                             { MakeError and ErrorCode    }
      END;                                                                      { in InstallInt.             }
  Temporary := 10;                                                              {-Temporary detects errors   }
  IF (ISize > MaxBuffSize) OR (OSize > MaxBuffSize)                             {-Turbo's max data structure }
  OR (ISize < 2) OR (OSize < 2)                                                 {-Minimum allowable buffer   }
    THEN Temporary := 9                                                         { size is two bytes.         }
    ELSE                                                                        {                            }
      IF (MAXAVAIL >= SIZEOF (PortType))                                        {-Check if enough free RAM   }
        THEN BEGIN                                                              { for port information table.}
               NEW (PortInfo [ID]);                                             {-Allocate port info table & }
               PortInfo [ID]^ := DefaultPortInfo;                               { initialize with defaults.  }
               IF (MAXAVAIL < ISize)                                            {-Enough input buffer RAM?   }
                 THEN DISPOSE (PortInfo [ID])                                   {-No, so deallocate table    }
                 ELSE                                                           {                            }
                   WITH PortInfo [ID]^ DO                                       {-Else allocate the input    }
                     BEGIN                                                      { buffer.                    }
                       GETMEM (I.Buffer, ISize);                                {                            }
                       IF (MAXAVAIL < OSize)                                    {-Check if enough free RAM   }
                         THEN                                                   { for the output buffer.     }
                           BEGIN                                                {                            }
                             FREEMEM (I.Buffer, ISize);                         {-If not, release the memory }
                             DISPOSE (PortInfo [ID]);                           { already allocated & exit.  }
                           END                                                  {                            }
                         ELSE                                                   {                            }
                           BEGIN                                                {                            }
                             GETMEM (O.Buffer, OSize);                          {-Else allocate the output   }
                             Temporary := 0;                                    { and clear the error flag.  }
                           END;                                                 {                            }
                     END;                                                       {                            }
             END;                                                               {                            }
  IF (Temporary <> 0)                                                           {-If any errors occurred in  }
    THEN                                                                        { allocating memory, create  }
      BEGIN                                                                     { an error condition & exit  }
        ID := -1;                                                               { even if error checking is  }
        PortInfo [ID] := NIL;                                                   { off.  This type of error   }
        {$IFDEF ErrorChecking}                                                  { is marked by returning a   }
          MakeError (Temporary, ID, ProcName);                                  { -1 port handle also.       }
        {$ENDIF}                                                                {                            }
        GOTO Abort;                                                             {                            }
      END;                                                                      {                            }
  IF (PortsInUse = 0)                                                           {-Install port timer before  }
    THEN InstallTimer;                                                          { installing IntHandler!     }
  INC (PortsInUse);                                                             {-Add newly installed port   }
  WITH PortInfo [ID]^ DO                                                        {-Everything is OK now.      }
    BEGIN                                                                       {                            }
      COMAddr := Addr;                                                          {-Initialize the port        }
      COMNmbr := Com;                                                           { information table values.  }
      IRQNmbr := IRQ;                                                           {                            }
      I.MaxSize := ISize;                                                       {                            }
      O.MaxSize := OSize;                                                       {                            }
      Temporary := PORT [COMAddr + MSR];                                        {                            }
      CTS := ($10 AND Temporary <> 0);                                          {-Detect the current state   }
      DSR := ($20 AND Temporary <> 0);                                          { of all port input lines.   }
      RI  := ($40 AND Temporary <> 0);                                          {                            }
      CD  := ($80 AND Temporary <> 0);                                          {                            }
      Temporary := PORT [COMAddr + LSR];                                        {-Clear pending interrupts.  }
      Temporary := PORT [COMAddr + RHR];                                        {                            }
      DisableInts;                                                              {-Accessing 8259 hardware    }
      Temporary := 1 SHL IRQ;                                                   {-If other port using same   }
      IF ((IRQMask AND Temporary) <> 0)                                         { IRQ then nothing must be   }
        THEN OldIntVector [ID] := OldIntVector [SameIRQ]                        { done to 8259 or vectors &  }
        ELSE                                                                    { save vector for this IRQ.  }
          BEGIN                                                                 {                            }
            IRQMask := IRQMask OR Temporary;                                    {-Update interrupt record    }
            GETINTVEC ($08 + IRQNmbr, OldIntVector [ID]);                       {-Save old interrupt vector  }
            SETINTVEC ($08 + IRQNmbr, @SerialInt);                              {-Install Comm_TP5 vector    }
            PORT [$21] := PORT [$21] AND NOT Temporary;                         {-Enable 8259 IRQ handling   }
          END;                                                                  {                            }
      Temporary := PORT [COMAddr + MCR];                                        {-Detect the current state   }
      RTS := ($02 AND Temporary <> 0);                                          { of all port output lines.  }
      DTR := ($01 AND Temporary <> 0);                                          {                            }
      PORT [COMAddr + MCR] := Temporary OR $08;                                 {-Enable 8250 interrupt line }
      PORT [COMAddr + LCR] := PORT [COMAddr + LCR] AND $7F;                     {-Set THR/RHR/IER active     }
      PORT [COMAddr + IER] := Mode;                                             {-Set interrupt types        }
      EnableInts;                                                               {                            }
    END;                                                                        {                            }
  {$IFDEF ErrorChecking}                                                        {                            }
    ErrorCode := 0;                                                             {-No errors occurred         }
  {$ENDIF}                                                                      {                            }
  Abort : InstallInt := ID;                                                     {-Always returns with some   }
END;                                                                            { port handler # or code.    }
                                                                                {                            }
PROCEDURE RemoveInt (ID : SHORTINT);                                            {-Uninstalls a port handler  }
VAR                                                                             { and releases allocated     }
  SameIRQ, Temporary : BYTE;                                                    { buffer memory.             }
BEGIN                                                                           {                            }
  {$IFDEF ErrorChecking}                                                        {                            }
    IF NOT ValidPort (ID, 'RemoveInt') THEN EXIT;                               {-Optional error trapping    }
  {$ENDIF}                                                                      {                            }
  DEC (PortsInUse);                                                             {-The handler is removed, &  }
  IF (PortsInUse = 0)                                                           { if it is the last one, the }
    THEN RemoveTimer;                                                           { port timer interrupt is    }
  WITH PortInfo [ID]^ DO                                                        { also removed.              }
    BEGIN                                                                       {                            }
      DisableInts;                                                              {-Accessing 8259 hardware    }
      PORT [COMAddr + MCR] := PORT [COMAddr + MCR] AND $F7;                     {-Disable 8250 interrupt line}
      PORT [COMAddr + LCR] := PORT [COMAddr + LCR] AND $7F;                     {-Set THR, THE & IER active  }
      PORT [COMAddr + IER] := $00;                                              {-Disable 8250 interrupts    }
      SameIRQ := ID;                                                            {                            }
      FOR Temporary := MaxPorts - 1 DOWNTO 0 DO                                 {-Find out if any other port }
        IF (Temporary <> ID) AND (PortInfo [Temporary] <> NIL)                  { handlers are using the IRQ }
        AND (PortInfo [Temporary]^.IRQNmbr = IRQNmbr)                           { assigned to this port.  If }
          THEN SameIRQ := Temporary;                                            { so, we should not attempt  }
      IF (SameIRQ <> ID)                                                        { to change the interrupt    }
        THEN                                                                    { vector used by the port    }
          BEGIN                                                                 { being removed.             }
            Temporary := 1 SHL IRQNmbr;                                         {-Get bit for IRQ mask       }
            PORT [$21] := PORT [$21] OR Temporary;                              {-Disable 8259 IRQ handling, }
            SETINTVEC ($08 + IRQNmbr, OldIntVector [ID]);                       { Replace original vector,   }
            IRQMask := IRQMask AND NOT Temporary;                               { and update IRQ mask.       }
          END;                                                                  {                            }
      EnableInts;                                                               {-Done with 8259 setup       }
      FREEMEM (O.Buffer, O.MaxSize);                                            {                            }
      FREEMEM (I.Buffer, I.MaxSize);                                            {-Return all allocated RAM   }
      DISPOSE (PortInfo [ID]);                                                  { to the heap and mark the   }
      PortInfo [ID] := NIL;                                                     { port handler "inactive".   }
    END;                                                                        {                            }
END;                                                                            {                            }
                                                                                {                            }
{$F+}                                                                           {-VERY IMPORTANT!  When the  }
PROCEDURE RemoveIntOnExit;                                                      { program quits normally or  }
BEGIN                                                                           { abnormally, the interrupt  }
  FOR Loop := 0 TO MaxPorts - 1 DO                                              { handlers are uninstalled   }
    IF (PortInfo [Loop] <> NIL)                                                 { when Turbo invokes this    }
      THEN RemoveInt (Loop);                                                    { exit procedure.            }
  EXITPROC := ExitSave;                                                         {-Return control to the      }
END;                                                                            { original exit procedure    }
{$F-}                                                                           {                            }
                                                                                {                            }
PROCEDURE SetRtsDtr (ID, NewRTS, NewDTR : SHORTINT);                            {-Changes RTS/DTR outputs    }
VAR                                                                             {                            }
  TempMCR : BYTE;                                                               {                            }
BEGIN                                                                           {                            }
  {$IFDEF ErrorChecking}                                                        { Optional error trapping    }
    IF NOT ValidPort (ID, 'SetRtsDtr') THEN EXIT;                               {-checks if port has been    }
  {$ENDIF}                                                                      { installed.                 }
  WITH PortInfo [ID]^ DO                                                        {                            }
    BEGIN                                                                       { UseDTR & UseRTS override a }
      DisableInts;                                                              { request to change the line }
      TempMCR := PORT [COMAddr + MCR];                                          { status. (ORD (FALSE) = 0). }
      CASE (NewDTR * ORD (UseDTR)) OF                                           {                            }
        -1 : TempMCR := TempMCR AND $FE;                                        {  0  leaves the line state  }
         1 : TempMCR := TempMCR OR $01;                                         {     unchanged.             }
      END;                                                                      {                            }
      CASE (NewRTS * ORD (UseRTS)) OF                                           {  1  sets the line "On",    }
        -1 : TempMCR := TempMCR AND $FD;                                        {     or at a  12V level.    }
         1 : TempMCR := TempMCR OR $02;                                         {                            }
      END;                                                                      { -1  sets the line "Off",   }
      PORT [COMAddr + MCR] := TempMCR;                                          {     or at a  12V signal.   }
      EnableInts;                                                               {                            }
      TempMCR := PORT [COMAddr + MCR];                                          {-DTR & RTS settings are     }
      DTR := (TempMCR AND $01) <> 0;                                            { confirmed by reading back  }
      RTS := (TempMCR AND $02) <> 0;                                            { the recently set values.   }
    END;                                                                        {                            }
END;                                                                            {                            }
                                                                                {                            }
PROCEDURE EmptyBuffer (ID : SHORTINT; TrueInFalseOut : BOOLEAN);                {-Forces In/Output buffers   }
VAR                                                                             { to the empty state. TRUE   }
  Loop : BYTE;                                                                  { clears in, FALSE for out.  }
BEGIN                                                                           {                            }
  FOR Loop := 0 to MaxPorts - 1 DO                                              {-A buffer number of -1 will }
    IF (ID = -1) OR (Loop = ID) AND (PortInfo [ID] <> NIL)                      { empty all buffers. Invalid }
      THEN                                                                      { buffer numbers do nothing. }
        WITH PortInfo [Loop]^ DO                                                {                            }
          BEGIN                                                                 {                            }
            DisableInts;                                                        {-Disable buffer activity    }
            IF TrueInFalseOut                                                   { while changing pointers.   }
              THEN                                                              {                            }
                BEGIN                                                           {                            }
                  I.Head := I.Tail;                                             {-Clear an input buffer      }
                  I.Size := 0;                                                  {                            }
                END                                                             {                            }
              ELSE                                                              {                            }
                BEGIN                                                           {                            }
                  O.Head := O.Tail;                                             {-Clear an output buffer     }
                  O.Size := 0;                                                  {                            }
                END;                                                            {                            }
            EnableInts;                                                         {-Reenable buffer activity   }
          END;                                                                  {                            }
END;                                                                            {                            }
                                                                                {                            }
PROCEDURE SetCOMPort (ID : SHORTINT; Baud, DBit, Prty, SBit : BYTE);            {-Set a port's communication }
VAR                                                                             { parameters.                }
  TempLCR : BYTE;                                                               {                            }
BEGIN                                                                           {                            }
  {$IFDEF ErrorChecking}                                                        {                            }
    IF NOT ValidPort (ID, 'SetCOMPort') THEN EXIT;                              {-Optional error trapping    }
  {$ENDIF}                                                                      { detects invalid port       }
  WITH PortInfo [ID]^ DO                                                        { handles.                   }
    BEGIN                                                                       {                            }
      DisableInts;                                                              {-Critical area              }
      TempLCR := PORT [COMAddr + LCR];                                          {                            }
      PORT [COMAddr + LCR] := TempLCR OR $80;                                   {-Set DLL & DLM active       }
      PORT [COMAddr + DLL] := LO (BaudTable [Baud]);                            {-Set the baud rate with the }
      PORT [COMAddr + DLM] := HI (BaudTable [Baud]);                            { predefined divisor values. }
      PORT [COMAddr + LCR] := (TempLCR AND $C0) OR ((DBit - 5) AND $03)         {-Set the other protocol     }
                              OR (((SBit - 1) SHL 2) AND $04)                   { attributes by writing LCR. }
                              OR ((Prty SHL 3) AND $38);                        { IER is also reenabled.     }
      EnableInts;                                                               {-End of critical area       }
    END;                                                                        {                            }
END;                                                                            {                            }
                                                                                {                            }
PROCEDURE GetCOMPort (ID : SHORTINT; VAR Baud, DBit, Prty, SBit : BYTE);        {-Detect the current port    }
VAR                                                                             { communication parameters.  }
  TempLCR : BYTE;                                                               {                            }
  TempBaud : WORD;                                                              {                            }
BEGIN                                                                           {                            }
  {$IFDEF ErrorChecking}                                                        {                            }
    IF NOT ValidPort (ID, 'GetCOMPort') THEN EXIT;                              {-Optional error trapping    }
  {$ENDIF}                                                                      { detects invalid port       }
  WITH PortInfo [ID]^ DO                                                        { handles.                   }
    BEGIN                                                                       {                            }
      DisableInts;                                                              {-Critical area              }
      TempLCR := PORT [COMAddr + LCR];                                          {                            }
      PORT [COMAddr + LCR] := TempLCR OR $80;                                   {-Set DLL & DLM active       }
      TempBaud := WORD (PORT [COMAddr + DLM]) SHL 8;                            {-Read baud rate divisor     }
      TempBaud := TempBaud + PORT [COMAddr + DLL];                              {                            }
      PORT [COMAddr + LCR] := TempLCR AND $7F;                                  {-Renable access to IER      }
      EnableInts;                                                               {-End of critical area       }
    END;                                                                        {                            }
  Baud := 255;                                                                  {-Set the port parameters    }
  DBit := (TempLCR AND $03) + 5;                                                { and search the known baud  }
  Prty := ((TempLCR SHR 3) AND $07);                                            { rate divisors.  If the     }
  SBit := ((TempLCR SHR 2) AND $01) + 1;                                        { baud rate divisor can not  }
  FOR TempLCR := 0 TO 9 DO                                                      { be identified, it will be  }
    IF (TempBaud = BaudTable [TempLCR])                                         { left 255 instead of being  }
      THEN Baud := TempLCR;                                                     { set 0 to 9 (corresponding  }
END;                                                                            { to elements of BaudName).  }
                                                                                {                            }
{$I Comm_IO5.PAS} { For simplicity, I/O routines are kept in a separate file. } {                            }
                                                                                {-Automatic initialization   }
BEGIN                                                                           {                            }
  ExitSave := EXITPROC;                                                         {-VERY IMPORTANT!  This lets }
  ExitProc := @RemoveIntOnExit;                                                 { the program halt safely.   }
  FOR Loop := 0 TO MaxPorts - 1 DO                                              {                            }
    PortInfo [Loop] := NIL;                                                     {-No handlers installed      }
  MaxPortsInUse := MaxPorts;
  PortsInUse := 0;                                                              {-No ports in use            }
  IRQMask := 0;                                                                 {-No IRQs used by Comm_TP5   }
  {$IFDEF ErrorChecking}                                                        {                            }
    ErrorCode := 0;                                                             {-Clear the optional error   }
    ErrorPort := 0;                                                             { flags for "No Errors".     }
    {$IFNDEF NoErrorMsgs}                                                       {                            }
      ShowMessages := TRUE;                                                     {-Error messages are enabled }
      ErrMsgX := 0;                                                             {-and will be displayed at   }
      ErrMsgY := 0;                                                             { the cursor position.       }
    {$ENDIF}                                                                    {                            }
  {$ENDIF}                                                                      {                            }
END.                                                                            {                            }
