PROGRAM Comm_TTY;

{                     Turbo Pascal (Version 5.0 and higher)                   } {-This is the comment column }
{                                                                             } { which explains the program }
{    Comm_TTY.PAS Ver. 4.00 - Demo for Comm_TP5.PAS Ver. 2.50 TPU Routines    } { operation where necessary. }
{                       (c) Copyright, 1989, 1990, 1991                       } {                            }
{                              Kevin R. Bulgrien                              } { It is likely to cause your }
{                               January 27, 1999                              } { hard copy to exceed a line }
{                                                                             } { width of 80 characters, so }
{ See the accompanying file Comm_TP5.DOC of specific information regarding    } { your printer will need to  }
{ distribution policies & supplementary documentation for the source code.    } { be set for smaller print.  }
{                                                                             } {                            }
{ Written by:  Kevin R. Bulgrien           Version 4.00 completed 08/06/91    } { I hope this is acceptably  }
{                                                                             } { inconvenient as I believe  }
{ Send all correspondance to:              Kevin R. Bulgrien                  } { that documentation is far  }
{                                          P. O. Box 563                      } { easier to read when it is  }
{ CompuServe: 102174,1561                  Longview, TX 75606                 } { not mingled with the code. }
{ Internet:   kbulgrien@worldnet.att.net   (903) 757-5163                     } {                            }
{                                                                             } { No code should be placed   }
{ Also contact via: c/o K. A. Bulgrien, 1700 Sheldon Road, Sandusky, MI 48471 } { in this column so that it  }
{                                                                             } { will be easier to remove.  }
{ This code package must always include the following files:  Comm_TP5.PAS,   } {                            }
{ Comm_TP5.H, Comm_TP5.ASM, Comm_TPA.OBJ, Comm_TPB.OBJ, Comm_TP5.DOC,         } {-When Comm_TPx.OBJ is here, }
{ Comm_IO5.PAS, Comm_IO5.H, and Comm_TTY.PAS.                                 } { no need for an assembler.  }
                                                                                {                            }
USES DOS, CRT, Comm_TP5 { Version 2.50 };                                       {                            }
                                                                                {                            }
TYPE                                                                            {                            }
  ComSettingsRecord = RECORD                                                    {-A structure to hold port   }
                       Baud : BYTE;                                             { communications settings.   }
                       Parity : BYTE;                                           {                            }
                       Stop : BYTE;                                             {                            }
                       Bits : BYTE;                                             {                            }
                     END;                                                       {                            }
                                                                                {                            }
  ComSettingsType = ARRAY [0..MaxPorts-1] OF ComSettingsRecord;                 {                            }
                                                                                {                            }
  CharSet = SET OF CHAR;                                                        {                            }
                                                                                {                            }
VAR                                                                             {                            }
  ComSettings : ComSettingsType;                                                {-Current port settings      }
  PortID, Select : SHORTINT;                                                    {-Port number currently set  }
  InKey : CHAR;                                                                 {-General purpose input      }
  Com : BYTE;                                                                   {-General purpose            }
                                                                                {                            }
PROCEDURE WriteHexWord (Value : WORD);                                          {-Provided to allow Comm_TTY }
CONST                                                                           { to print the port base     }
  Hex : STRING [16] = '0123456789ABCDEF';                                       { address in hex.            }
BEGIN                                                                           {                            }
  WRITE (Hex [HI (Value) DIV 16 + 1], Hex [HI (Value) MOD 16 + 1]);             {                            }
  WRITE (Hex [LO (Value) DIV 16 + 1], Hex [LO (Value) MOD 16 + 1]);             {                            }
END;                                                                            {                            }
                                                                                {                            }
FUNCTION ReadChar (ValidChars : CharSet; UpperCase : BOOLEAN) : CHAR;           {-Lets program specify the   }
VAR                                                                             { allowable input keys and   }
  InkeyChar : CHAR;                                                             { whether or not the input   }
BEGIN                                                                           { is to be forced to upper   }
  REPEAT                                                                        { case.                      }
    InkeyChar := READKEY;                                                       {                            }
    IF UpperCase                                                                {                            }
      THEN InkeyChar := UPCASE (InkeyChar);                                     { Wait until a valid key is  }
  UNTIL (InkeyChar IN ValidChars);                                              {-pressed.                   }
  IF (InkeyChar > ' ')                                                          {                            }
    THEN WRITE (InkeyChar);                                                     {-Do not try to print input  }
  ReadChar := InkeyChar;                                                        { if it was a control        }
END;                                                                            { character.                 }
                                                                                {                            }
FUNCTION FindCOMPortHandler (Com : BYTE) : SHORTINT;                            {-Searches all of the port   }
VAR                                                                             { handlers and returns the   }
  Loop, ID : SHORTINT;                                                          { handler number which was   }
BEGIN                                                                           { installed for the given    }
  ID := -1;                                                                     { COM port.  If one was not  }
  FOR Loop := 0 TO MaxPorts - 1 DO                                              { found, an error message    }
    IF (PortInfo [Loop] <> NIL) AND (PortInfo [Loop]^.COMNmbr = Com)            { is displayed, and a port   }
      THEN ID := Loop;                                                          { number of -1 is returned.  }
  IF (ID < 0)                                                                   {                            }
    THEN                                                                        {                            }
      BEGIN                                                                     {                            }
        GOTOXY (ErrMsgX, ErrMsgY);                                              {                            }
        WRITE ('ERROR:  COM', Com, ' not installed');                           {                            }
        ErrorCode := 255;                                                       {                            }
      END;                                                                      {                            }
  FindCOMPortHandler := ID;                                                     {                            }
END;                                                                            {                            }
                                                                                {                            }
PROCEDURE ErrorHandler;                                                         {-Asks user for a keypress   }
BEGIN                                                                           { so we know the error       }
  WRITE ('.  Press <Esc> to continue...');                                      { message was seen.          }
  IF (ReadChar ([#27], FALSE) = #27) THEN;                                      {                            }
END;                                                                            {                            }
                                                                                {                            }
{ A crude but effective procedure to allow the user to change settings of a serial port in use. PortID &     }
{ ComSettings determine how the port is currently set up. As the parameters are changed, ComSettings is also }
{ updated.  Once again, keep in mind that the object of this program is not to provide a glamorous terminal  }
{ program.  Rather it serves as an example for those wanting to use serial routines in their own programs.   }
                                                                                {                            }
PROCEDURE SetUpPort (ID : SHORTINT);                                            {                            }
VAR                                                                             {                            }
  ResetPort : BOOLEAN;                                                          {-TRUE when settings changed }
  InKey : CHAR;                                                                 {                            }
  Loop : BYTE;                                                                  {                            }
BEGIN                                                                           {                            }
  ResetPort := FALSE;                                                           {-Default of no changes made }
  WRITELN (#10, 'COM', PortInfo [ID]^.COMNmbr, ' Setup', #10);                  {                            }
  FOR Loop := 0 TO 9 DO                                                         {-Display all possible baud  }
    BEGIN                                                                       { rates supported by the     }
      WRITE (Loop, '=', BaudName [Loop]);                                       { Comm_TP5 unit.             }
      IF (Loop <> 9)                                                            {                            }
        THEN WRITE (', ')                                                       {                            }
        ELSE WRITELN;                                                           {                            }
    END;                                                                        {                            }
  FOR Loop := 0 TO 7 DO                                                         {-Display all the possible   }
    BEGIN                                                                       { parity types supported     }
      WRITE (Loop, '=', PrtyName [Loop]);                                       { by the Comm_TP5 unit.      }
      IF (Loop <> 7)                                                            {                            }
        THEN WRITE (', ')                                                       {                            }
        ELSE WRITELN (#10);                                                     {                            }
    END;                                                                        {                            }
  WRITE ('Baud rate [', ComSettings [ID].Baud, ']? ');                          {-Select a baud rate.        }
  InKey := ReadChar (['0'..'9', #13], FALSE);                                   {                            }
  IF (InKey <> #13)                                                             {                            }
    THEN BEGIN                                                                  {                            }
           ComSettings [ID] . Baud := ORD (InKey) - 48;                         {                            }
           ResetPort := TRUE;                                                   {                            }
         END;                                                                   {                            }
  WRITE (#13, #10, 'Parity    [', ComSettings [ID].Parity, ']? ');              {-Select a parity setting.   }
  InKey := ReadChar (['0'..'7', #13], FALSE);                                   {                            }
  IF (InKey <> #13)                                                             {                            }
    THEN BEGIN                                                                  {                            }
           ComSettings [ID] . Parity := ORD(InKey) - 48;                        {                            }
           ResetPort := TRUE;                                                   {                            }
         END;                                                                   {                            }
  WRITE (#13, #10, 'Data bits [', ComSettings [ID] . Bits, ']? ');              {-Select number of data bits }
  InKey := ReadChar (['5'..'8', #13], FALSE);                                   {                            }
  IF (InKey <> #13)                                                             {                            }
    THEN BEGIN                                                                  {                            }
           ComSettings [ID] . Bits := ORD(InKey) - 48;                          {                            }
           ResetPort := TRUE;                                                   {                            }
         END;                                                                   {                            }
  WRITE (#13, #10, 'Stop bits [', ComSettings [ID].Stop, ']? ');                {-Select number of stop bits }
  InKey := ReadChar (['1'..'2', #13], FALSE);                                   {                            }
  IF (InKey <> #13)                                                             {                            }
    THEN BEGIN                                                                  {                            }
           ComSettings [ID] . Stop := ORD(InKey) - 48;                          {                            }
           ResetPort := TRUE;                                                   {                            }
         END;                                                                   {                            }
  IF ResetPort                                                                  {-If the settings changed,   }
    THEN SetCOMPort (ID, ComSettings [ID] . Baud,                               { reset the port.            }
                         ComSettings [ID] . Bits,                               {                            }
                         ComSettings [ID] . Parity,                             {                            }
                         ComSettings [ID] . Stop);                              {                            }
  WRITELN (#10);                                                                {                            }
END;                                                                            {                            }
                                                                                {                            }
{ This provides a simple terminal emulation that might be used to prove that Comm_TP5 really works, and that }
{ it is not hard to use.  I got to playing, and perhaps it got a bit more complex than necessary...          }
                                                                                {                            }
PROCEDURE TTY;                                                                  {                            }
VAR                                                                             {                            }
  Carrier,                                                                      {-Temporary to hold CD value }
  ExitTTY,                                                                      {-TRUE when ready to quit    }
  DataReady,                                                                    {-TRUE if buffer not empty   }
  LocalEcho,                                                                    {-Local echo current setting }
  ShowHelp : BOOLEAN;                                                           {-TRUE if need help screen   }
  OldCarrier : ARRAY [0..MaxPorts-1] OF BOOLEAN;                                {-Helps detect carrier change}
  Loop : SHORTINT;                                                              {-General purpose loop       }
  Buffer : CHAR;                                                                {-A character buffer         }
BEGIN                                                                           {                            }
  FOR Loop := 0 TO MaxPorts-1 DO                                                {-Force Carrier status to be }
    IF (PortInfo [Loop] <> NIL)                                                 { displayed the first time   }
      THEN BEGIN                                                                { a port is logged.          }
             DisableInts;                                                       {                            }
             OldCarrier [Loop] := NOT PortInfo [Loop]^.CD;                      {                            }
             EnableInts;                                                        {                            }
           END;                                                                 {                            }
  LocalEcho := FALSE;                                                           {-Set defaults               }
  ShowHelp := TRUE;                                                             {-Show help screen on start  }
  ExitTTY := FALSE;                                                             {                            }
  CLRSCR;                                                                       {                            }
  REPEAT                                                                        {-Terminal emulation starts  }
    WITH PortInfo [PortID]^ DO                                                  {-Point to logged port       }
      BEGIN                                                                     { handler info table.        }
        IF ShowHelp                                                             {-Print the help screen if   }
          THEN                                                                  { it has been requested.     }
            BEGIN                                                               {                            }
              WRITELN (#13, #10, 'Terminal emulator commands', #10);            {-Brief summary of command   }
              WRITE   ('Alt-H  Help screen    ');                               {                            }
              WRITE   ('Alt-E  Local Echo On/Off      ');                       {                            }
              WRITELN ('Press ALT and a number key');                           {                            }
              WRITE   ('Alt-X  Exit           ');                               {                            }
              WRITE   ('Alt-P  Set Port Parameters    ');                       {                            }
              WRITELN ('to log to another COM port');                           {                            }
              ShowHelp := FALSE;                                                {                            }
            END;                                                                {                            }
        DisableInts;                                                            {                            }
        DataReady := (I.Size > 0);                                              {-If data has been received, }
        EnableInts;                                                             { print one character.       }
        IF DataReady                                                            {                            }
          THEN                                                                  {                            }
            BEGIN                                                               { CHR(12) is interpreted as  }
              IF IReadCOM (PortID, Buffer) THEN ;                               { a FormFeed, and so clears  }
              CASE Buffer OF                                                    { the screen.                }
                #12 : CLRSCR;                                                   {                            }
                ELSE  WRITE (Buffer);                                           {                            }
              END;                                                              {                            }
            END;                                                                {                            }
        DisableInts;                                                            {                            }
        Carrier := CD;                                                          {                            }
        EnableInts;                                                             {                            }
        IF (OldCarrier [PortID] <> Carrier)                                     {-If a change in carrier     }
          THEN                                                                  { detect occurs, notify the  }
            BEGIN                                                               { user, and log the change.  }
              IF Carrier                                                        {                            }
                THEN WRITELN (#10,#13,'CARRIER DETECTED (COM', COMNmbr, ')')    {                            }
                ELSE WRITELN (#10,#13,'NO CARRIER (COM', COMNmbr, ')');         {                            }
              OldCarrier [PortID] := Carrier;                                   {                            }
            END;                                                                {                            }
        IF KEYPRESSED                                                           {-If a key is pressed, then  }
          THEN                                                                  { process it.                }
            BEGIN                                                               {                            }
              Buffer := READKEY;                                                {                            }
              IF (Buffer = #0)                                                  {-Extended key codes require }
                THEN                                                            { another read               }
                  BEGIN                                                         {                            }
                    Buffer := READKEY;                                          {                            }
                    CASE Buffer OF                                              {                            }
                      #120..                                                    {-<F1> through <F4> log on   }
                      #123: BEGIN                                               { to COM1 through COM4, but  }
                              Loop := FindCOMPortHandler (BYTE (Buffer) - 119); { only if installed.         }
                              IF (Loop >= 0)                                    {                            }
                                THEN                                            {                            }
                                  BEGIN                                         {                            }
                                    PortID := Loop;                             {                            }
                                    WRITE (#13, #10, #10, 'COM');               {                            }
                                    WRITELN (PortInfo [PortID]^.COMNmbr, #10);  {                            }
                                  END;                                          {                            }
                            END;                                                {                            }
                      #45 : ExitTTY := TRUE;                                    {-<ALT X> exits the program  }
                      #35 : ShowHelp := TRUE;                                   {-<ALT H> shows help screen  }
                      #25 : SetupPort (PortID);                                 {-<ALT P> allows port setup  }
                      #18 : LocalEcho := NOT LocalEcho;                         {-<ALT E> toggles Local Echo }
                      ELSE  IF IWriteCOM (PortID, #0 + Buffer) THEN ;           {-Other extended key codes   }
                    END;                                                        { are sent to the port       }
                  END                                                           {                            }
                ELSE                                                            {                            }
                  BEGIN                                                         {-Normal key codes are sent  }
                     IF IWriteCOM (PortID, Buffer) THEN ;                       {-FormFeed clears screen if  }
                     CASE Buffer OF                                             { or translated and sent     }
                       #12 : IF LocalEcho THEN CLRSCR;                          { local echo is on           }
                       #13 : IF LocalEcho THEN WRITELN;                         {                            }
                       ELSE  IF LocalEcho THEN WRITE (Buffer);                  {                            }
                     END;                                                       {                            }
                  END;                                                          {                            }
            END;                                                                {                            }
      END;                                                                      {                            }
  UNTIL ExitTTY;                                                                {-Continue emulation until   }
END;                                                                            { <ALT X> is pressed.        }
                                                                                {                            }
BEGIN                                                                           {                            }
  ErrMsgY := 25;                                                                {-All Comm_TP5 errors will   }
  ErrMsgX := 1;                                                                 { be displayed at (1,25).    }
  Select := -1;                                                                 {-No ports selected.         }
  REPEAT                                                                        {                            }
    CLRSCR;                                                                     {                            }
    WRITELN ('COMM_TTY Terminal Emulator', #10);                                {                            }
    WRITELN ('Serial ports:', #10);                                             {                            }
    FOR PortID := 0 TO MaxPorts-1 DO                                            {-List all of the ports that }
      WITH PortInfo [PortID]^ DO                                                { the Comm_TTY can be set up }
        BEGIN                                                                   { to handle, along with the  }
          WRITE ('Handler #', PortID, ':  ');                                   { base addresses and IRQs of }
          IF PortInfo [PortID] = NIL                                            { the ports.  Also display   }
            THEN WRITE ('Not Installed')                                        { the current status of      }
            ELSE BEGIN                                                          { the port handlers.         }
                   WRITE ('COM', COMNmbr, ', Address: ');                       {                            }
                   WriteHexWord (COMAddr);                                      {                            }
                   WRITE (', IRQ', IRQNmbr, ', 2400 baud, 8N1');                {                            }
                   IF Select = PortID                                           {                            }
                     THEN WRITE (', Selected');                                 {                            }
                 END;                                                           {                            }
          WRITELN;                                                              {                            }
        END;                                                                    {                            }
    WRITELN (#10, 'A port must be installed & selected in order to begin.');    {-Display the command prompt }
    WRITE (#10, '<F1> Install port, <F3> Remove port, <F5> Select port, ');     {                            }
    WRITE ('<F7> TTY, or <F9> Quit? ');                                         {                            }
    InKey := ReadChar ([#0, #27], FALSE);                                       {-Wait for a command key     }
    IF (InKey=#0) THEN InKey := READKEY;                                        {                            }
    GOTOXY (1,25);                                                              {                            }
    CASE InKey OF                                                               {                            }
      #59 : BEGIN                                                               {-<F1> Install a port handler}
              WRITE ('<Esc> to cancel.  Install which COM port (1-4)? ');       {                            }
              InKey := ReadChar ([#27, '1'..'4'], FALSE);                       {                            }
              WRITE (#13);                                                      {                            }
              CLREOL;                                                           {                            }
              IF (InKey <> #27)                                                 {                            }
                THEN                                                            {                            }
                  BEGIN                                                         {                            }
                    Com := BYTE (InKey) - 48;                                   {                            }
                    PortID := InstallInt (Receive+ModemStatus,                  {-Install the port handler   }
                                          Com,                                  { to monitor input lines and }
                                          COMIRQNmbr [Com],                     { receive data automatically.}
                                          COMAddress [COM], 1024, 100);         { Only standard port settings}
                    IF (ErrorCode = 0)                                          { are supported by Comm_TTY. }
                      THEN                                                      {                            }
                        BEGIN                                                   { Some modems do not work    }
                          SetRtsDtr (PortID, 1, 1);                             {-without RTS and DTR active.}
                          SetCOMPort (PortID, ORD (B2400), 8, ORD (None), 1);   {-For simplicity, we will    }
                          ComSettings [PortID].Baud := ORD (B2400);             { only allow the port to be  }
                          ComSettings [PortID].Bits := 8;                       { initially set up to 2400,  }
                          ComSettings [PortID].Parity := ORD (None);            { 8N1.  The baud rate, data  }
                          ComSettings [PortID].Stop := 1;                       { bits, parity, & stop bits  }
                        END;                                                    { can be adjusted after the  }
                    IF (ErrorCode = 0)                                          { TTY emulation begins.      }
                      THEN Select := PortID                                     {                            }
                      ELSE ErrorHandler;                                        {-Last installed port is     }
                  END;                                                          { automatically "selected".  }
            END;                                                                {                            }
      #61 : BEGIN                                                               {-<F3> removes port handlers }
              WRITE ('<Esc> to cancel.  Uninstall which COM port (1-4)? ');     {                            }
              InKey := ReadChar ([#27, '1'..'4'], FALSE);                       { A port handler may only be }
              IF (InKey <> #27)                                                 { removed if it has been     }
                THEN                                                            { installed previously.      }
                  BEGIN                                                         {                            }
                    PortID := FindCOMPortHandler (BYTE (InKey) - 48);           {                            }
                    RemoveInt (PortID);                                         {                            }
                    IF (ErrorCode <> 0)                                         {                            }
                      THEN ErrorHandler;                                        {                            }
                    IF (PortID = Select)                                        {                            }
                      THEN Select := -1;                                        {                            }
                  END;                                                          {                            }
            END;                                                                {                            }
      #63 : BEGIN                                                               {-<F5> selects a port to be  }
              WRITE ('<Esc> to cancel.  Select which COM port (1-4)? ');        { logged to when the TTY     }
              InKey := ReadChar ([#27, '1'..'4'], FALSE);                       { emulation begins.          }
              IF (InKey <> #27)                                                 {                            }
                THEN                                                            { It is only possible to     }
                  BEGIN                                                         { select one port, and it    }
                    PortID := FindCOMPortHandler (BYTE (InKey) - 48);           { must be already installed. }
                    IF (PortID >= 0)                                            {                            }
                      THEN Select := PortID                                     {                            }
                      ELSE ErrorHandler;                                        {                            }
                  END;                                                          {                            }
            END;                                                                {                            }
      #65 : IF (Select >= 0)                                                    {-<F7> begins the TTY        }
              THEN                                                              { emulation.                 }
                BEGIN                                                           {                            }
                  PortID := Select;                                             {                            }
                  TTY;                                                          {                            }
                END;                                                            {                            }
      #27 : Inkey := #67;                                                       {-<F9> quits Comm_TTY        }
    END;                                                                        {                            }
  UNTIL (InKey = #67);                                                          {                            }
END.                                                                            {                            }
